/*****************************************************************************
* "Gifinfo"								     *
*									     *
* Written by:  Martin Edlman			Ver 1.0.1, Jul. 1999	     *
******************************************************************************
* Program to display information (size and comments) in GIF file.	     *
* uses libungif	on Linux. Not tried on other systems.			     *
* Options:								     *
* -h : on-line help.							     *
* -f : format output string						     *
*      \h - height							     *
*      \w - width							     *
*      \c - comment							     *
*      \f - file name							     *
*      Example: -f 'Image \f:\n Size: \wx\h\n Comment: \c\n'		     *
******************************************************************************
* History:								     *
* 21 Jun 99 - Version 1.0.1 by Martin Edlman.				     *
*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <gif_lib.h>

#define DEFAULTFORMAT   "Size: \\wx\\h\\nComment: \\c\\n"

void usage(const char *msg) {
    printf("gifinfo: Display information from gif file\n", msg);
    printf("Usage: gifinfo [-f Format] [-h] filename\n");
    printf("Format: quoted text string used for formating of information\n");
    printf("  Special characters for various information can be used\n");
    printf("  \\f - filename of GIF file\n");
    printf("  \\c - comment\n");
    printf("  \\w - width of image\n");
    printf("  \\h - height of image\n");
    printf("  \\n - new line\n");
    printf("  \\\\ - backslash\n");
    printf("Default format will be used if not specified with -f:\n");
    printf("  %s\n\n", DEFAULTFORMAT);
    printf("%s\n", msg);
    exit(1);
}

int main(int argc, char *argv[])
{

    int	i, size, width, height, extcode;
    GifRecordType recordtype;
    GifByteType  *extension;
    char         *filename = NULL, *comment = NULL, *format = NULL;
    GifRowType    rowbuffer;
    GifFileType  *gft;

    if (argc >= 2) {
        for(i = 1; i < argc; i++) {
            if(argv[i][0] == '-') {
                switch(argv[i][1]) {
                case 'h':  usage(""); break;
                case 'f':  if(strlen(argv[i]) > 2) {
                               format = &argv[i][2];
                           }
                           else if(argc > i+1) {
                               format = argv[++i];
                           }
                           else {
                               usage("Error: Format not specified");
                           }
                           break;
                default:   break;
                }
            }
            else {
                filename = argv[i];
            }
        }
    }

    if(format == NULL) {
        format = malloc(strlen(DEFAULTFORMAT)+1);
        strcpy(format, DEFAULTFORMAT);
    }

    if (filename) {
	if ((gft = DGifOpenFileName(filename)) == NULL)
            usage("Error: cannot open file");
    }
    else {
	if ((gft = DGifOpenFileHandle(0)) == NULL)
            usage("Error: cannot read from stdin");
    }

    comment = (char*) malloc(1);
    strcpy(comment, "\0");

    size = gft->SWidth * sizeof(GifPixelType);           /* Size in bytes one row.*/
    if ((rowbuffer = (GifRowType) malloc(size)) == NULL) /* Thrash buffer for reading image. */
	    usage("Error: Cannot allocate memory for trash buffer");

    /* Scan the content of the GIF file and load the image(s) in: */
    do {
	if (DGifGetRecordType(gft, &recordtype) == GIF_ERROR) {
	    PrintGifError();
	    exit(-1);
	}
	switch (recordtype) {
	    case IMAGE_DESC_RECORD_TYPE:
		if (DGifGetImageDesc(gft) == GIF_ERROR) {
		    PrintGifError();
		    exit(-1);
		}
		width = gft->Image.Width;
		height = gft->Image.Height;
                /* We don't care about Interlaced as image is not interesting, read sequentialy */
                for (i = 0; i < height; i++) {
                    if (DGifGetLine(gft, &rowbuffer[0], width) == GIF_ERROR) {
                        PrintGifError();
                        exit(-1);
                    }
                }
                break;
	    case EXTENSION_RECORD_TYPE:
		/* Skip any extension blocks in file except comments */
		if (DGifGetExtension(gft, &extcode, &extension) == GIF_ERROR) {
		    PrintGifError();
		    exit(-1);
		}
		while (extension != NULL) {
		    if(extcode == COMMENT_EXT_FUNC_CODE) {
			extension[extension[0]+1] = '\000';   /* Convert gif's pascal-like string */
			comment = (char*) realloc(comment, strlen(comment) + extension[0] + 1);
			strcat(comment, (char*)extension+1);
		    }
		    if (DGifGetExtensionNext(gft, &extension) == GIF_ERROR) {
			PrintGifError();
			exit(-1);
		    }
		}
		break;
	    case TERMINATE_RECORD_TYPE:
		break;
	    default:		    /* Should be traps by DGifGetRecordType. */
		break;
	}
    }
    while (recordtype != TERMINATE_RECORD_TYPE);

    while (format[0] != '\0') {
        if (format[0] == '\\') {
            format++;
            switch (format[0]) {
            case 'w': printf("%i",  width);     break;
            case 'h': printf("%i",  height);    break;
            case 'c': printf("%s",  comment);   break;
            case 'f': printf("%s", *filename);  break;
            case 'n': printf("\n");             break;
            default:  printf("%c",  format[0]); break;
            }
         }
         else {
            printf("%c", format[0]);
         }
         format++;
    }

    if (DGifCloseFile(gft) == GIF_ERROR) {
	PrintGifError();
	exit(-1);
    }
}
